=begin
  Class for converting task hashes to strings and vice versa
=end
class Ztodo::Converter
  
  # Convert string representation of task to hash  
  def str_to_hash str
    out = {}
    data = str.split("\n")
    out[:key] = parse_key_line data[0]
    out[:priority] = parse_priority_line data[1]
    out[:done] = parse_completeness_line data[2]
    out[:description] = extract_description data.clone
    out
  end

  # Convert hash representation of task to string
  def hash_to_str hash
    out = ''
    out += hash[:key].to_s+" # unique identifier of task\n"
    out += format_priority(hash[:priority])+" # priority: 'low', 'normal' or 'high'\n"
    out += (hash[:done] ? 'done' : 'undone' )+" # task completeness: 'done' or 'undone'\n"
    out += hash[:description].to_s
    out
  end

  def colored_priority int_val
    require 'colorize'
    if int_val == Ztodo::Project::LOW
      'low'.green
    elsif int_val == Ztodo::Project::NORMAL
      'normal'.yellow
    elsif int_val == Ztodo::Project::HIGH
      'high'.red
    else
      raise Exception.new('Priority should be -1, 0 or 1. Given: '+int_val)
    end
  end

  protected 

  # Format priority integer value as string
  def format_priority int_val
    if int_val == Ztodo::Project::LOW
      'low'
    elsif int_val == Ztodo::Project::NORMAL
      'normal'
    elsif int_val == Ztodo::Project::HIGH
      'high'
    else
      raise Exception.new('Priority should be -1, 0 or 1. Given: '+int_val)
    end
  end

  # Parse line with unique identifier
  def parse_key_line line
    line = clear_line line    
    if line.nil? || line.empty? 
      'task'+Random.rand(1000).to_s
    else 
      line
    end
  end

  # Parse line with priority data
  def parse_priority_line line
    return Ztodo::Project::NORMAL if line.nil? || line.empty?
    line = clear_line line    

    if line == 'low'
      Ztodo::Project::LOW    
    elsif line == 'high'
      Ztodo::Project::HIGH
    else # 'normal' and any other option
      Ztodo::Project::NORMAL
    end
  end

  # Parse line with completeness data
  def parse_completeness_line line    
    line = clear_line line    
    return line == 'done'
  end


  # Return line without commented part
  def clear_line line
    return nil if line.nil?

    if line.index('#').nil?
      res = line
    else
      res = line[0,line.index('#')]
    end
    res.strip!
    return res
  end

  # Return description data
  def extract_description splitted_data    
    return '' if splitted_data.size<=3
    3.times { splitted_data.slice! 0 }    
    splitted_data.join "\n"
  end

end