require 'yaml'

=begin
  Represent Project object.  
=end
class Ztodo::Project
  
  attr_reader :loaded

  LOW = -1
  NORMAL = 0
  HIGH = 1

  def initialize
    @loaded = false
    @data = {}
  end

  # Init project file (.ztodo) in current directory
  def init!
    @data = {}
    save!
  end

  # load project from file in current directory
  def load!
    raise Exception.new('ztodo project file is not exists') unless File.exists?(Dir.pwd+'/.ztodo')

    @data = YAML::load(File.open(Dir.pwd+'/.ztodo'))
    @loaded = true
  end

  # Save project to file in current directory
  def save!    
    File.open(Dir.pwd + '/.ztodo', 'w+') { |f| f.write(@data.to_yaml) }
    @loaded = true
  end

  # Return all tasks  
  def tasks show_all = FALSE
    return @data.reject {|k,v| v[:done]} if !show_all    
    @data.clone
  end

  # Add task
  # Raise en exception if key is already used
  def add task
    raise Exception.new('Such key already exists') if @data.include?(task[:key].to_sym)
    @data[task[:key].to_sym] = task
  end

  # Modify task
  # Raise en exception if there is no task with such key
  def modify key, task
    raise Exception.new('No task with such key') unless @data.include?(key.to_sym)
    @data.delete key
    add task
  end

  # Remove task
  # Raise en exception if there is no task with such key
  def remove key
    raise Exception.new('No task with such key') unless @data.include?(key.to_sym)
    @data.delete key
  end

  # Create task (but not add it to data)
  def create
    new_key = 0
    
    @data.each do |k, elem| 
      begin            
        curr = elem[:key].to_s.to_i
        new_key = curr if new_key < curr
      rescue
      end
    end

    new_key += 1
    {:key=>new_key.to_s, :description=>'put description here', 
      :done=>false, :priority=> Ztodo::Project::NORMAL}    
  end  

end